{
    "id": "aramislab/{{ pipeline.command_name }}",
    "author": "John Doe",
    "version": "0.1.0",
    "dependencies": [
        {
            "type": "binary",
            "name": "clinica",
            "version": ">=0.1.0"
        }
    ]
}
